@@ -16,7 +16,7 @@ from functools import reduce
 
 from bravado.client import SwaggerClient
 
-__version__ = "0.1.21"
+__version__ = "0.1.35"
 
 try:
     import io
@@ -68,6 +68,9 @@ def log(*args,**kwargs):
 class Empty(Exception):
     pass
 
+class CorruptEntry(Exception):
+    pass
+
 class CurrentTaskUnfinished(Exception):
     pass
 
@@ -131,7 +134,7 @@ class Task:
                 for i, e_l in enumerate(entry.splitlines()):
                     print(f"problematic json: {i:5d}", e_l)
                 open("/tmp/problematic_entry.json", "wt").write(entry)
-                raise
+                raise CorruptEntry(e, entry)
         else:
             task_dict = entry
 
@@ -189,7 +192,7 @@ class Task:
         if 'object_identity' in self.task_data:
             short_name = self.task_data['object_identity']['full_name']
 
-        logger.warning("generating key %s short name %s", key, short_name)
+        logger.debug("generating key %s short name %s", key, short_name)
 
         return key
 
@@ -199,6 +202,51 @@ class Task:
     def filename_instance(self):
         return "unset"
 
+    def score_worker_knowledge(self, worker_knowledge) -> float:
+        score = 1.
+
+        if worker_knowledge is not None:
+            for r in worker_knowledge:
+                if len(r) != 1:
+                    raise RuntimeError(f"each worker_knowledge entry needs to have one operation, found {r}")
+
+                op, kv = list(r.items())[0]
+
+                if op == "any-of":
+                    entry_score = 0.
+                elif op == "none-of":
+                    entry_score = 1.
+
+                for l in kv:
+                    pt = l['key']
+                    s = l['value']
+
+                    logger.info("scoring worker knowledge: %s %s %s, current score %s", op, pt, s, entry_score)
+                
+                    s_d = json.loads(
+                                json.dumps(reduce(lambda D,x:D[x], pt, self.task_data)).replace('null', '"None"')
+                            )
+                    logger.info("scoring on task data selection %s", s_d)
+
+                    if op == "any-of":
+                        if s in s_d:
+                            entry_score += 1.
+                    elif op == "none-of":
+                        if s in s_d:
+                            entry_score *= 0.
+                    else:
+                        raise RuntimeError(f"unknown operation {op} in score_worker_knowledge")
+
+                    logger.info("now score %s", score)
+                
+                score *= entry_score
+                logger.info("entry score %s, total score %s", entry_score, score)
+
+                if score <= 0:
+                    break
+
+        return score
+
 def makedir_if_neccessary(directory):
     try:
         os.makedirs(directory)
@@ -319,7 +367,7 @@ class Queue:
 
         if len(r) > 1:
             raise RuntimeError(f"found multiple entries for key {key}: suspecting database inconsistency!")
-        
+
         if len(r) == 0:
             return None
 
@@ -331,7 +379,6 @@ class Queue:
         return r
 
 
-    
     def put(self, task_data: dqtyping.TaskData, submission_data=None, depends_on=None) -> Union[dqtyping.TaskEntry, None]:
         logger.info("putting in queue task_data %s", task_data)
 
@@ -376,12 +423,15 @@ class Queue:
             logger.debug("task entry: %s", d)
             return d
 
-        if depends_on is None:
-            self.insert_task_entry(task, "waiting")
-            log("task inserted as waiting")
-        else:
-            self.insert_task_entry(task, "locked")
-            log("task inserted as locked")
+        def insert_it():
+            if depends_on is None:
+                self.insert_task_entry(task, "waiting")
+                log("task inserted as waiting")
+            else:
+                self.insert_task_entry(task, "locked")
+                log("task inserted as locked")
+        
+        insert_it()
 
         instance_for_key = None
         for i in range(5):
@@ -392,6 +442,7 @@ class Queue:
             except Exception as e:
                 logger.warning("race condition?")
                 time.sleep(2.)
+                insert_it()
 
         if instance_for_key is None:
             raise Exception("inserted task does not exist!")
@@ -433,43 +484,49 @@ class Queue:
     def get_worker_states(self):
         return [ model_to_dict(r) for r in EventLog.select().where(EventLog.worker_state!="unset").order_by(EventLog.timestamp.desc()).limit(1).execute(database=None) ]
 
-    def get(self, update_expected_in_s: float=-1):
-        ""
-        if self.current_task is not None:
-            raise CurrentTaskUnfinished(self.current_task)
+    def get_one_task(self, update_expected_in_s, offset):
+        select_task = TaskEntry.select(TaskEntry.key)\
+                    .where( (TaskEntry.state=="waiting") & (TaskEntry.queue==self.queue) )\
+                    .order_by(TaskEntry.created)\
+                    .offset(offset)\
+                    .limit(1)
 
-        self.note_worker_state("ready")
+        r = select_task.execute(database=None) # not atomic!?
+        
+        if len(r) == 0:
+            raise Empty()
 
-        max_update_expected_in_s = 3600
-        default_update_expected_in_s = 1200
-
-        if update_expected_in_s > max_update_expected_in_s:
-            logger.warning("update expected timeout %s is too long, setting to maximum %s", update_expected_in_s, max_update_expected_in_s)
-            update_expected_in_s = max_update_expected_in_s
-
-        if update_expected_in_s <= 0:
-            logger.warning("no update expected timeout, setting to default %s", default_update_expected_in_s)
-            update_expected_in_s = default_update_expected_in_s
-    
-        r=TaskEntry.update({
+        t = TaskEntry.update({
                         TaskEntry.state:"running",
                         TaskEntry.worker_id:self.worker_id,
                         TaskEntry.modified:datetime.datetime.now(),
                         TaskEntry.update_expected_in_s:update_expected_in_s
                     })\
-                    .order_by(TaskEntry.created)\
-                    .where( (TaskEntry.state=="waiting") & (TaskEntry.queue==self.queue) ).limit(1).execute(database=None)
+                    .where(TaskEntry.key == r[0].key)
+
+        print("task update sql:", t.sql())
+
+        r = t.execute(database=None)
 
-        if r==0:
-            #self.try_all_locked()
-            raise Empty()
 
         entries=TaskEntry.select().where(TaskEntry.worker_id==self.worker_id,TaskEntry.state=="running").order_by(TaskEntry.modified.desc()).limit(1).execute(database=None)
         if len(entries)>1:
             raise Exception(f"several tasks ({len(entries)}) are running for this worker: impossible!")
 
         entry=entries[0]
-        self.current_task=Task.from_task_dict(entry.task_dict_string)
+
+        try:
+            self.current_task = Task.from_task_dict(entry.task_dict_string)
+        except CorruptEntry:
+            r = TaskEntry.update({
+                        TaskEntry.modified:datetime.datetime.now(),
+                        TaskEntry.state: "corrupt",
+                    }).where(TaskEntry.key == entry.key).execute(database=None)
+
+            logger.error("found corrupt entry %s, marking as so", entry.key)
+            return None
+
+        # validate
         self.current_task_stored_key=self.current_task.key
 
         if self.current_task.key != entry.key:
@@ -493,6 +550,53 @@ class Queue:
 
             raise Exception("Inconsistent storage")
 
+    def get(self, update_expected_in_s: float=-1, worker_knowledge=None):
+        ""
+        if self.current_task is not None:
+            raise CurrentTaskUnfinished(self.current_task)
+
+        self.note_worker_state("ready")
+
+        max_update_expected_in_s = 3600
+        default_update_expected_in_s = 1200
+
+        if update_expected_in_s > max_update_expected_in_s:
+            logger.warning("update expected timeout %s is too long, setting to maximum %s", update_expected_in_s, max_update_expected_in_s)
+            update_expected_in_s = max_update_expected_in_s
+
+        if update_expected_in_s <= 0:
+            logger.warning("no update expected timeout, setting to default %s", default_update_expected_in_s)
+            update_expected_in_s = default_update_expected_in_s
+    
+        offset = 0
+        while True:
+            self.get_one_task(update_expected_in_s, offset=offset)
+
+            if self.current_task is None:
+                time.sleep(1)
+                continue
+
+            worker_fit_score = self.current_task.score_worker_knowledge(worker_knowledge) # also sort TODO
+            if worker_fit_score <= 0:
+                logger.warning("picked task %s has non-positive (%s) worker (%s) score: skipping",
+                        self.current_task.key, worker_fit_score, worker_knowledge)
+
+                r=TaskEntry.update({
+                                TaskEntry.state:"waiting",
+                            })\
+                            .where( (TaskEntry.key == self.current_task.key) ).limit(1).execute(database=None)
+
+                offset += 1
+                self.current_task = None
+
+                time.sleep(1)
+                continue
+
+            break
+
+        if self.current_task is None:
+            raise Empty()
+
 
         log("task is running",self.current_task)
         self.current_task_status = "running"
@@ -507,14 +611,28 @@ class Queue:
         # compatibility
         return self.clear_event_log()
 
-    def clear_event_log(self, only_older_than_days: Union[float,None]=None, only_kind: Union[str,None]=None):
+    def clear_event_log(self, 
+                        only_older_than_days: Union[float,None]=None, 
+                        only_kind: Union[str,None]=None,
+                        leave_last: Union[int,None]=None):
         ""
-        if only_older_than_days is None and only_kind is None:
+        if only_older_than_days is None and only_kind is None and leave_last is None:
             logger.warning('this is very desctructive: clearing all event log')
             return EventLog.delete().execute(database=None)
 
         f = []
 
+        if leave_last:
+            i_last = EventLog.select().order_by(EventLog.id.desc()).limit(1).execute(database=None)
+
+            if len(i_last) == 0:
+                logger.warning('no events left, can not clean')
+            else:
+                i_last = i_last[0].id
+                logger.warning('last event %d, clearing all but %s', i_last, leave_last)
+                f.append(EventLog.id < i_last - leave_last)
+                logger.warning('clearing older events: %s', f)
+
         if only_older_than_days:
             t0 = datetime.datetime.now() - datetime.timedelta(days=only_older_than_days)
 
@@ -531,11 +649,14 @@ class Queue:
             else:
                 raise RuntimeError(f"unknown kind {only_kind}; expecting 'task' or 'worker'")
 
-        F = reduce(lambda x,y: x&y, f)
+        if len(f) > 0:
+            F = reduce(lambda x,y: x&y, f)
 
-        N = EventLog.delete().where(F).execute(database=None)
+            N = EventLog.delete().where(F).execute(database=None)
 
-        logger.info("clearing: %s", N)
+            logger.info("clearing: %s", N)
+        else:
+            N = 0
 
         return N
     
@@ -776,6 +897,10 @@ class Queue:
 
 
     def task_done(self):
+        if self.current_task is None:
+            log("WARNING: trying to claim done task, but no task is current")
+            return
+
         log("task done, closing:",self.current_task.key,self.current_task)
         log("task done, stored key:",self.current_task_stored_key)
 
