@@ -31,7 +31,6 @@ class QueueProxy(DataFacts, Queue):
         return self.client.hub.version().response().result
 
     def list_queues(self, pattern):
-        print(self.client.queues.list().response().result)
         return [ QueueProxy(self.leader+"@"+q) for q in self.client.queues.list().response().result ]
 
     def find_task_instances(self,task,klist=None):
@@ -52,9 +51,15 @@ class QueueProxy(DataFacts, Queue):
 
         return r
 
-    def clear_event_log(self, only_older_than_days: Union[float,None]=None, only_kind: Union[str,None]=None):
+    def clear_event_log(self, 
+                        only_older_than_days: Union[float,None]=None, 
+                        only_kind: Union[str,None]=None,
+                        leave_last: Union[int,None]=None, 
+                        ):
         return self.client.log.clear(only_older_than_days=only_older_than_days,
-                                     only_kind=only_kind).response().result
+                                     only_kind=only_kind,
+                                     leave_last=leave_last,
+                                     ).response().result
     
     def view_log(self, task_key=None, since=0):
         if task_key is None:
@@ -124,11 +129,14 @@ class QueueProxy(DataFacts, Queue):
                 ).response().result
 
 
-    def get(self, update_expected_in_s=-1):
+    def get(self, update_expected_in_s=-1, worker_knowledge=None):
         if self.current_task is not None:
             raise CurrentTaskUnfinished(self.current_task)
 
-        r = self.client.worker.getOffer(worker_id=self.worker_id, queue=self.queue, update_expected_in_s= update_expected_in_s).response()
+        r = self.client.worker.getOffer(worker_id=self.worker_id, 
+                                        queue=self.queue, 
+                                        update_expected_in_s= update_expected_in_s, 
+                                        worker_knowledge_json=json.dumps(worker_knowledge or {})).response()
 
         if r.result is None:
             raise Empty()
