@@ -128,6 +128,8 @@ def test_one():
     task_log =  queue.view_log()
     
     print("complete log", len(task_log))
+    x = queue.clear_event_log(leave_last=50)
+    print("clear last 10", x)
 
     n = queue.clear_event_log(only_older_than_days=-1, only_kind="task")
     print("clear of task", n)
@@ -137,6 +139,9 @@ def test_one():
 
     x = queue.clear_event_log()
     print("final clear", x)
+    
+    x = queue.clear_event_log(leave_last=10)
+    print("clear last 10", x)
 
     assert x == 0
     assert n + m == len(task_log)
@@ -337,3 +342,50 @@ def test_expiration():
     assert queue.info['running'] == 0
     assert queue.info['failed'] == 1
 
+def test_score():
+    import dqueue
+    
+    queue=dqueue.Queue("test-queue")
+    queue.wipe(["waiting","done","running","failed","locked"])
+    queue.clear_task_history()
+    
+    assert queue.info['waiting']==0
+    assert queue.info['done']==0
+    assert queue.info['running']==0
+    assert queue.info['failed']==0
+    assert queue.info['locked']==0
+
+
+    t11 = dict(test=1,
+              data=dict(
+                        modules=["osa10-module","osa11-module"]
+                   )
+             )
+    assert queue.put(t11)['state']=="submitted"
+
+    t10 = dict(test=1,
+              data=dict(
+                        modules=["osa10-module"]
+                   )
+             )
+    assert queue.put(t10)['state']=="submitted"
+
+
+    tr11=queue.get(worker_knowledge=[
+            {'any-of': [
+                    dict(key=['data', 'modules'], value='osa11-module')
+                 ]},
+         ]).task_data
+    queue.task_done()
+    
+    assert t11 == tr11
+    
+    tr10=queue.get(worker_knowledge=[
+        {'none-of': [
+                    dict(key=['data', 'modules'], value='osa11-module')
+                 ]},
+         ]).task_data
+    queue.task_done()
+
+    assert t10 == tr10
+    
